#include "MMA8452.h"
#include "uart.h"
#include <stdbool.h>


unsigned char AccData[6];
short Xout_14_bit, Yout_14_bit, Zout_14_bit;
float Xout_g, Yout_g, Zout_g;
char DataReady;
char Xoffset, Yoffset, Zoffset;
unsigned char Reg;
unsigned char Transient_Event = 0;
unsigned char Mode = 0;




// The SparkFun breakout board defaults to 1, set to 0 if SA0 jumper on the bottom of the board is set
#define SA0 1
#if SA0
#define MMA8452_ADDRESS 0x1D  // SA0 is high, 0x1C if low
#else
#define MMA8452_ADDRESS 0x1C
#endif

// Read i registers sequentially, starting at address into the dest byte array
void readRegisters(byte address, int i, byte * dest)
{
  int j=0;

  i2cSendStart();
  i2cWaitForComplete();

  i2cSendByte((MMA8452_ADDRESS<<1)); // write 0xB4
  i2cWaitForComplete();

  i2cSendByte(address);	// write register address
  i2cWaitForComplete();

  i2cSendStart();
  i2cSendByte((MMA8452_ADDRESS<<1)|0x01); // write 0xB5
  i2cWaitForComplete();
  for (j=0; j<i; j++)
    {
      i2cReceiveByte(true);
      i2cWaitForComplete();
      dest[j] = i2cGetReceivedByte(); // Get MSB result
    }
  i2cWaitForComplete();
  i2cSendStop();

  cbi(TWCR, TWEN); // Disable TWI
  sbi(TWCR, TWEN); // Enable TWI
}

// Read a single byte from address and return it as a byte
byte readRegister(uint8_t address)
{
  byte data;


  i2cSendStart();
  i2cWaitForComplete();

  i2cSendByte((MMA8452_ADDRESS<<1)); // Write 0xB4
  i2cWaitForComplete();

  i2cSendByte(address);	// Write register address
  i2cWaitForComplete();

  i2cSendStart();

  i2cSendByte((MMA8452_ADDRESS<<1)|0x01); // Write 0xB5
  i2cWaitForComplete();
  i2cReceiveByte(true);
  i2cWaitForComplete();

  data = i2cGetReceivedByte();	// Get MSB result
  i2cWaitForComplete();
  i2cSendStop();

  cbi(TWCR, TWEN);	// Disable TWI
  sbi(TWCR, TWEN);	// Enable TWI

  return data;
}

// Writes a single byte (data) into address
void writeRegister(unsigned char address, unsigned char data)
{
  i2cSendStart();
  i2cWaitForComplete();

  i2cSendByte((MMA8452_ADDRESS<<1)); // Write 0xB4
  i2cWaitForComplete();

  i2cSendByte(address);	// Write register address
  i2cWaitForComplete();

  i2cSendByte(data);
  i2cWaitForComplete();

  i2cSendStop();
  i2cWaitForComplete();
}


// Sets the MMA8452 to standby mode.
// It must be in standby to change most register settings
void MMA8452Standby()
{
  byte c = readRegister(0x2A);
  writeRegister(0x2A, c & ~(0x01));
}

// Sets the MMA8452 to active mode.
// Needs to be in this mode to output data
void MMA8452Active()
{
  byte c = readRegister(0x2A);
  writeRegister(0x2A, c | 0x01);
}


void readAccelData(int * destination)
{
  int i=0;
  byte rawData[6];  // x/y/z accel register data stored here

  readRegisters(0x01, 6, &rawData[0]);  // Read the six raw data registers into data array

  // Loop to calculate 12-bit ADC and g value for each axis
  for (i=0; i<6; i+=2)
    {
      destination[i/2] = ((rawData[i] << 8) | rawData[i+1]) >> 4;  // Turn the MSB and LSB into a 12-bit value
      if (rawData[i] > 0x7F)
	{
	  // If the number is negative, we have to make it so manually (no 12-bit data type)
	  destination[i/2] -= 0x1000;
	}
    }
}

// Initialize the MMA8452 registers
// See the many application notes for more info on setting all of these registers:
// http://www.freescale.com/webapp/sps/site/prod_summary.jsp?code=MMA8452Q
// Feel free to modify any values, these are settings that work well for me.
void initMMA8452(void)
{
  byte val,val1=1;

  MMA8452Standby();  // Must be in standby to change registers

  // Set up the full scale range to 2, 4, or 8g.
  writeRegister(XYZ_DATA_CFG, 0x00);

  while(val!=val1) {
    val = (SMODS_LOW_POW |  MODS_NORMAL_POW | SLPE);
    writeRegister(CTRL_REG2,val);
    val1= readRegister(CTRL_REG2);
  }

  val=0;
  val1=1;

  while(val!=val1) {
    val=ASLP_COUNT_50_HZ_960_MS;
    writeRegister(ASLP_COUNT, val);
    val1= readRegister(ASLP_COUNT);
  }

  val=0;
  val1=1;

  while(val!=val1) {

    val = WAKE_FF_MT;
    writeRegister(CTRL_REG3, val);  
    val1= readRegister(CTRL_REG3);
  }

  val=0;
  val1=1;

  while(val!=val1){
    val = (INT_EN_FF_MT);
    writeRegister(CTRL_REG4,  val); 
    val1= readRegister(CTRL_REG4);
  }

  val=0;
  val1=1;

  while(val!=val1) {
    val = (INT_CFG_FF_MT);
    writeRegister(CTRL_REG5,val);
    val1= readRegister(CTRL_REG5);
  }

  val=0;
  val1=1;

  //SETUP THE MOTION DETECTION
  while(val!=val1) {
    val = (ELE_MOTION | OAE_MOTION | XYZ_EFE);
    writeRegister(FF_MT_CFG , val);  
    print("FFT_MT_CFG :");
  }                              

  val=0;
  val1=1;

  while(val!=val1) {
    val=THS_0_2_G;
    writeRegister(FF_MT_THS,val);
    

    val1= readRegister(FF_MT_THS);
  }

  writeRegister(FF_MT_COUNT, FF_MT_DEBOUNCE);    
	

  val=0;
  val1=1;

  while(val!=val1) {
    val=(ASLP_50_HZ | ODR_100_HZ | ACTIVE | LNOISE);
    writeRegister(CTRL_REG1, val);
    val1= readRegister(CTRL_REG1);
  }

  MMA8452Active();  
}


void setup_MMA8452()
{
  byte c;

  i2cInit();

  // Set up the interrupt pins, they're set as active high, push-pull
  pinMode(int1Pin, INPUT);
  digitalWrite(int1Pin, LOW);

  // Read the WHO_AM_I register, this is a good test of communication
  c = readRegister(0x0D);  // Read WHO_AM_I register

  if (c == 0x2A) // WHO_AM_I should always be 0x2A
    {
      initMMA8452();  // init the accelerometer if communication is OK
    
      c=readRegister(FF_MT_SRC);

    }
  
}
