#include "SPI.h"
#include "EEPROM.h"
#include "lib_aci.h"
#include "aci_setup.h"
#include "uart_over_ble.h"
#include "uart.h"
#include "MMA8452.h"

#include <avr/sleep.h>
#include <avr/power.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

/**
 * Initialise la puce BLE avec le header "service.h"
 */
#include "services.h"


/**
 * Variables pour la communication Blutooth
 */
#ifdef SERVICES_PIPE_TYPE_MAPPING_CONTENT
static services_pipe_type_mapping_t
services_pipe_type_mapping[NUMBER_OF_PIPES] = SERVICES_PIPE_TYPE_MAPPING_CONTENT;
#else
#define NUMBER_OF_PIPES 0
static services_pipe_type_mapping_t * services_pipe_type_mapping = NULL;
#endif

static const hal_aci_data_t setup_msgs[NB_SETUP_MESSAGES] PROGMEM = SETUP_MESSAGES_CONTENT;
static hal_aci_evt_t  aci_data;
static volatile byte ack = 0;

/* Buffer pour l'envoi et la reception de donnees */
#define MAX_TX_BUFF 64
static uint8_t tx_buff[MAX_TX_BUFF];
static uint8_t tx_buffer_len = 0;

#define MAX_RX_BUFF 64

static uint8_t rx_buff[MAX_RX_BUFF+1];
static uint8_t rx_buffer_len = 0;
static uint8_t *p_before = &rx_buff[0] ;
static uint8_t *p_back = &rx_buff[0];

/* Define how assert should function in the BLE library */
void __ble_assert(char *file, uint16_t line)
{
  print("ERROR ");
  print(file);
  print(": ");
  print_int(line);
  while(1);
}

/* Variables Globales pour le comptage du nombre de pas*/
uint16_t pas;
int mode_connect =0;

void setup(void)
{
  // Initialisation de l'interruption pour le nombre de pas
  EICRA |= (1 << ISC00);    
  EIMSK |= (1 << INT0);    
  
  /**
     Initialise tout les services du BLE grace au header service.h
  */
  if (NULL != services_pipe_type_mapping)
    {
      aci_state.aci_setup_info.services_pipe_type_mapping = &services_pipe_type_mapping[0];
    }
  else
    {
      aci_state.aci_setup_info.services_pipe_type_mapping = NULL;
    }
  aci_state.aci_setup_info.number_of_pipes    = NUMBER_OF_PIPES;
  aci_state.aci_setup_info.setup_msgs         = (hal_aci_data_t*)setup_msgs;
  aci_state.aci_setup_info.num_setup_msgs     = NB_SETUP_MESSAGES;

  /**
     Initialise les broches de l'atmega
  */
  aci_state.aci_pins.board_name = BOARD_DEFAULT; 
  aci_state.aci_pins.reqn_pin   = 9; 
  aci_state.aci_pins.rdyn_pin   = 3; 
  aci_state.aci_pins.mosi_pin   = 11;
  aci_state.aci_pins.miso_pin   = 12;
  aci_state.aci_pins.sck_pin    = 13;

  aci_state.aci_pins.spi_clock_divider      = SPI_CLOCK_DIV8;

  aci_state.aci_pins.reset_pin              = 4; 
  aci_state.aci_pins.active_pin             = UNUSED;
  aci_state.aci_pins.optional_chip_sel_pin  = UNUSED;
  aci_state.aci_pins.interface_is_interrupt     = false;
  aci_state.aci_pins.interrupt_number       = 3;
  
  lib_aci_init(&aci_state, false);
}

/**
 * Detecte les evenement du BLE
 */
void aci_loop()
{
  
  static bool setup_required = false;
  int i=0;
 
  if (lib_aci_event_get(&aci_state, &aci_data))
    {
      aci_evt_t * aci_evt;
      aci_evt = &aci_data.evt;
      switch(aci_evt->evt_opcode)
	{
	case ACI_EVT_DEVICE_STARTED: //BLE Connecte
	  {
	    aci_state.data_credit_total = aci_evt->params.device_started.credit_available;
	    switch(aci_evt->params.device_started.device_mode)
	      {
	      case ACI_DEVICE_SETUP: //BLE Initialise
		setup_required = true;
		break;

	      case ACI_DEVICE_STANDBY:
		if (aci_evt->params.device_started.hw_error)
		  {
		    _delay_ms(20); 
		  }
		else
		  {
		    lib_aci_connect(0/* in seconds : 0 means forever */, 0x0050 /* advertising interval 50ms*/);              		  
		  }

		break;
	      default:
		break;
	      }
	  }
	  break; //BLE pret a la reception et l envoi de donnees

	case ACI_EVT_CMD_RSP:
	  if (ACI_CMD_GET_DEVICE_VERSION == aci_evt->params.cmd_rsp.cmd_opcode)
	    {
	      lib_aci_set_local_data(&aci_state, PIPE_DEVICE_INFORMATION_HARDWARE_REVISION_STRING_SET,
				     (uint8_t *)&(aci_evt->params.cmd_rsp.params.get_device_version), sizeof(aci_evt_cmd_rsp_params_get_device_version_t));
	    }
	  break;

	case ACI_EVT_CONNECTED: //BLE connecte au telephone
	  mode_connect = 1;
	  aci_state.data_credit_available = aci_state.data_credit_total;
	  lib_aci_device_version();
	  break;

	case ACI_EVT_PIPE_STATUS:
	  if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX))
	    {
	      lib_aci_change_timing_GAP_PPCP();   
	    }
	  break;

	case ACI_EVT_TIMING:
          lib_aci_set_local_data(&aci_state,
				 PIPE_UART_OVER_BTLE_UART_LINK_TIMING_CURRENT_SET,
				 (uint8_t *)&(aci_evt->params.timing.conn_rf_interval),
				 PIPE_UART_OVER_BTLE_UART_LINK_TIMING_CURRENT_SET_MAX_SIZE);
        
	  break;

	case ACI_EVT_DISCONNECTED: //BLE deconnecte
	  //Attente d une nouvelle connection
	  lib_aci_connect(0/* in seconds */, 0x050 /* advertising interval 50ms*/);
			
	  break;


	case ACI_EVT_DATA_RECEIVED:  //Reception de Donnee
	  if (PIPE_UART_OVER_BTLE_UART_RX_RX == aci_evt->params.data_received.rx_data.pipe_number)
	    {
              if(rx_buffer_len == MAX_RX_BUFF)
		{
		  break;
		}
	      else
		{
		  if(p_back == &rx_buff[MAX_RX_BUFF])
		    {
		      p_back = &rx_buff[0];
		    }
		  *p_back = aci_evt->params.data_received.rx_data.aci_data[i];
		  rx_buffer_len++;
		  p_back++;
		}                    
             
	    }
	  break;

	case ACI_EVT_DATA_CREDIT:
	  aci_state.data_credit_available = aci_state.data_credit_available + aci_evt->params.data_credit.credit;
	  ack=1;
        
	  break;

	case ACI_EVT_PIPE_ERROR:
	  if (ACI_STATUS_ERROR_PEER_ATT_ERROR != aci_evt->params.pipe_error.error_code)
	    {
	      aci_state.data_credit_available++;
	    }
	  break;

	case ACI_EVT_HW_ERROR:
	  //Erreur lors de la connection, attente d une nouvelle connection
	  lib_aci_connect(0/* in seconds, 0 means forever */, 0x0050 /* advertising interval 50ms*/);
	  println("Advertising started. Tap Connect on the nRF UART app");
	  break;
	default:
	  break;
	}
    }
  else
    {
      /**
       *Pas d evenement, attente d un nouvel evenement
       */
    }

  if(setup_required)
    {
      if (SETUP_SUCCESS == do_aci_setup(&aci_state))
	{
	  setup_required = false;
	}
    }
}


/**
 * Fonction pour envoyer les donnees
 * Les donnee sont stockes dans un tableau de taille 64 puis envoyer par le BLE avec la fonction ble_do_events
 */
void ble_write(unsigned char data)
{
  if(tx_buffer_len == MAX_TX_BUFF)
    {
      return;
    }
  tx_buff[tx_buffer_len] = data;
  tx_buffer_len++;
}

void ble_write_bytes(unsigned char *data, uint8_t len)
{
  int i=0;
  for (i = 0; i < len; i++)
    ble_write(data[i]);
}

/**
 * Lit les donnees recu par le BLE
 * Les donnees reçut sont stockes dans un tableau de taille 64
 * Il suffit de faire un appel a la fonction pour recuperer les donnees reçut
 */
int ble_read()
{
  int data;
  if(rx_buffer_len == 0) return -1;
  if(p_before == &rx_buff[MAX_RX_BUFF])
    {
      p_before = &rx_buff[0];
    }
  data = *p_before;
  p_before ++;
  rx_buffer_len--;
  return data;
}

unsigned char ble_connected()
{
  return mode_connect;
}

unsigned char ble_busy()
{
  if(digitalRead(aci_state.aci_pins.reqn_pin) == HIGH)
    {
      return 0;
    }
  else
    {
      return 1;
    }
}

/**
 * Cette fonction permet de generer l envois de 
 * toutes les donnees stockes dans le tableau de la fonction ble_write()
 */
void ble_do_events()
{
  if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX))
    {
      if(tx_buffer_len > 0)
        {
	  unsigned char Index = 0;
	  while(tx_buffer_len > 20)
            {
	      lib_aci_send_data(PIPE_UART_OVER_BTLE_UART_TX_TX, &tx_buff[Index], 20);
	      tx_buffer_len -= 20;
	      Index += 20;
	      aci_state.data_credit_available--;
	      ack = 0;
	      while (!ack)
		aci_loop();
            }
		
	  lib_aci_send_data(PIPE_UART_OVER_BTLE_UART_TX_TX,& tx_buff[Index], tx_buffer_len);
	  tx_buffer_len = 0;
	  aci_state.data_credit_available--;
	  ack = 0;
	  while (!ack)
	    aci_loop();
        }
    }
  aci_loop();
}

/**
 * Fonction pour recuperer la temperature du capteur TMP36GZ
 * Ajoute les valeurs du capteur dans le buffer a envoyer
 */
void send_temp(void) {
  unsigned int value=0;
  float temp =0;
	
  ad_init(0);
  value=ad_sample(); //temperature sur 10bits
  temp=(3.3*value)/255; //temperature en volt
  temp=(temp*25)/0.75; //temperature en °C
	
  ble_write((int)temp);

  temp=temp-(int)temp;
  temp=temp*100;
	
  ble_write((int)temp);
}

/**
 * Cette fonction simule le cardiofrequencemetre que nous devions ajouter a notre tee-shirt
 */
void send_frequence(uint8_t* freq) {
  *freq+=2;
  if(*freq==200) *freq=150; 
  ble_write(*freq);
}

/**
 * Ajoute les valeurs du nombre pas dans le buffer sous la forme : millier, centaine, dizaine, unite
 */
void send_pas() {
  int millier=0,centaine=0,dizaine=0,unit=0;
  millier=pas/1000;
  centaine=(pas-millier*1000)/100;
  dizaine=(pas-(millier*1000)-(centaine*100))/10;
  unit=pas-(millier*1000)-(centaine*100)-(dizaine*10);
		
  ble_write(millier);
  ble_write(centaine);
  ble_write(dizaine);
  ble_write(unit);
	
}

/**
 * Fonction d'interruption qui recupere l interruption engendre par l accelerometre et incremente le compteur pas
 */
ISR (INT0_vect)
{
  pas++;
  cli();
}


 
int main() {
		
  //Definition des varibles 
  uint8_t frequence=100;
  pas=0;
	
  //Initialisation des registres de l accelerometre
  setup_MMA8452();
	
  //Initialisation du BLE
  setup();
	
  //Boucle infini pour l envoi des donnees	
  while(1) {
    sei();
		
    //Recupere les evenement du BLE
    aci_loop();
  		
    /**
     * Si le BLE est connecte, alors on envoit les donnees
     * Pour l'envoi des donnees, on commence par envoyer 0xC0
     * On envoit ensuite la temperature sur deux octets, le premier octets correspond aux entiers et le deuxieme aux decimales
     * Puis la frequence sur un octet
     * Le nombre de pas sur 4 octets avec les milliers, les centaines, les dizaines puis les unites
     * Puis on finit par un octet 0xC1
     */
  		 
    if(ble_connected()) {
			
      ble_write(0xC0); 
			
      send_temp();
					
      send_frequence(&frequence);
									
      send_pas();
						
      ble_write(0xC1);
			
      //On appelle ble_do_events pour realiser l envoi du buffer contenant toutes les donnees
      ble_do_events();
    }  		
		
  }
  return 0;
}
